@@ -30,7 +30,21 @@ class ContactData(object):
             names[0]: contact.b1,
             names[1]: contact.b2
         }
+        self._normal = contact.normal
         self._point = contact.pos
+        self._depth = contact.penetration_depth
+
+    @property
+    def normal(self):
+        """
+        The 3D intersection normal for this contact.
+
+        Returns
+        -------
+        normal : (3,) float
+          The intersection normal.
+        """
+        return self._normal
 
     @property
     def point(self):
@@ -44,6 +58,18 @@ class ContactData(object):
         """
         return self._point
 
+    @property
+    def depth(self):
+        """
+        The penetration depth of the 3D point of intersection for this contact.
+
+        Returns
+        -------
+        depth : float
+          The penetration depth.
+        """
+        return self._depth
+
     def index(self, name):
         """
         Returns the index of the face in contact for the mesh with
@@ -57,7 +83,7 @@ class ContactData(object):
         Returns
         -------
         index : int
-          The index of the face in collison
+          The index of the face in collision
         """
         return self._inds[name]
 
@@ -145,16 +171,13 @@ class CollisionManager(object):
         Initialize a mesh-mesh collision manager.
         """
         if fcl is None:
-            raise ValueError('No FCL Available!')
+            raise ValueError('No FCL Available! Please install the python-fcl library')
         # {name: {geom:, obj}}
         self._objs = {}
         # {id(bvh) : str, name}
         # unpopulated values will return None
         self._names = collections.defaultdict(lambda: None)
 
-        # cache BVH objects
-        # {mesh.md5(): fcl.BVHModel object}
-        self._bvh = {}
         self._manager = fcl.DynamicAABBTreeCollisionManager()
         self._manager.setup()
 
@@ -185,19 +208,20 @@ class CollisionManager(object):
         if transform.shape != (4, 4):
             raise ValueError('transform must be (4,4)!')
 
-        # create or recall from cache BVH
-        bvh = self._get_BVH(mesh)
+        # create BVH/Convex
+        geom = self._get_fcl_obj(mesh)
+
         # create the FCL transform from (4,4) matrix
         t = fcl.Transform(transform[:3, :3], transform[:3, 3])
-        o = fcl.CollisionObject(bvh, t)
+        o = fcl.CollisionObject(geom, t)
 
         # Add collision object to set
         if name in self._objs:
             self._manager.unregisterObject(self._objs[name])
         self._objs[name] = {'obj': o,
-                            'geom': bvh}
+                            'geom': geom}
         # store the name of the geometry
-        self._names[id(bvh)] = name
+        self._names[id(geom)] = name
 
         self._manager.registerObject(o)
         self._manager.update()
@@ -276,10 +300,12 @@ class CollisionManager(object):
         if transform is None:
             transform = np.eye(4)
 
-        # Create FCL data
-        b = self._get_BVH(mesh)
+        # create BVH/Convex
+        geom = self._get_fcl_obj(mesh)
+
+        # create the FCL transform from (4,4) matrix
         t = fcl.Transform(transform[:3, :3], transform[:3, 3])
-        o = fcl.CollisionObject(b, t)
+        o = fcl.CollisionObject(geom, t)
 
         # Collide with manager's objects
         cdata = fcl.CollisionData()
@@ -297,13 +323,13 @@ class CollisionManager(object):
         if return_names or return_data:
             for contact in cdata.result.contacts:
                 cg = contact.o1
-                if cg == b:
+                if cg == geom:
                     cg = contact.o2
                 name = self._extract_name(cg)
 
                 names = (name, '__external')
                 if cg == contact.o2:
-                    names = reversed(names)
+                    names = tuple(reversed(names))
 
                 if return_names:
                     objs_in_collision.add(name)
@@ -429,7 +455,7 @@ class CollisionManager(object):
                     objs_in_collision.add(names)
                 if return_data:
                     if reverse:
-                        names = reversed(names)
+                        names = tuple(reversed(names))
                     contact_data.append(ContactData(names, contact))
 
         if return_names and return_data:
@@ -473,17 +499,21 @@ class CollisionManager(object):
         if transform is None:
             transform = np.eye(4)
 
-        # Create FCL data
-        b = self._get_BVH(mesh)
+        # create BVH/Convex
+        geom = self._get_fcl_obj(mesh)
 
+        # create the FCL transform from (4,4) matrix
         t = fcl.Transform(transform[:3, :3], transform[:3, 3])
-        o = fcl.CollisionObject(b, t)
+        o = fcl.CollisionObject(geom, t)
 
         # Collide with manager's objects
-        ddata = fcl.DistanceData()
+        ddata = fcl.DistanceData(fcl.DistanceRequest(enable_signed_distance=True))
         if return_data:
             ddata = fcl.DistanceData(
-                fcl.DistanceRequest(enable_nearest_points=True),
+                fcl.DistanceRequest(
+                    enable_nearest_points=True,
+                    enable_signed_distance=True
+                ),
                 fcl.DistanceResult()
             )
 
@@ -495,14 +525,14 @@ class CollisionManager(object):
         name, data = None, None
         if return_name or return_data:
             cg = ddata.result.o1
-            if cg == b:
+            if cg == geom:
                 cg = ddata.result.o2
 
             name = self._extract_name(cg)
 
             names = (name, '__external')
             if cg == ddata.result.o2:
-                names = reversed(names)
+                names = tuple(reversed(names))
             data = DistanceData(names, ddata.result)
 
         if return_name and return_data:
@@ -535,10 +565,13 @@ class CollisionManager(object):
         data : DistanceData
           Extra data about the distance query
         """
-        ddata = fcl.DistanceData()
+        ddata = fcl.DistanceData(fcl.DistanceRequest(enable_signed_distance=True))
         if return_data:
             ddata = fcl.DistanceData(
-                fcl.DistanceRequest(enable_nearest_points=True),
+                fcl.DistanceRequest(
+                    enable_nearest_points=True,
+                    enable_signed_distance=True,
+                ),
                 fcl.DistanceResult()
             )
 
@@ -590,10 +623,13 @@ class CollisionManager(object):
         data : DistanceData
           Extra data about the distance query
         """
-        ddata = fcl.DistanceData()
+        ddata = fcl.DistanceData(fcl.DistanceRequest(enable_signed_distance=True))
         if return_data:
             ddata = fcl.DistanceData(
-                fcl.DistanceRequest(enable_nearest_points=True),
+                fcl.DistanceRequest(
+                    enable_nearest_points=True,
+                    enable_signed_distance=True,
+                ),
                 fcl.DistanceResult()
             )
 
@@ -615,7 +651,7 @@ class CollisionManager(object):
 
             dnames = tuple(names)
             if reverse:
-                dnames = reversed(dnames)
+                dnames = tuple(reversed(dnames))
             data = DistanceData(dnames, ddata.result)
 
         if return_names and return_data:
@@ -627,22 +663,26 @@ class CollisionManager(object):
         else:
             return distance
 
-    def _get_BVH(self, mesh):
+    def _get_fcl_obj(self, mesh):
         """
-        Get a BVH for a mesh.
+        Get a BVH or Convex for a mesh.
 
         Parameters
         -------------
         mesh : Trimesh
-          Mesh to create BVH for
+          Mesh to create BVH/Convex for
 
         Returns
         --------------
-        bvh : fcl.BVHModel
-          BVH object of source mesh
+        obj : fcl.BVHModel or fcl.Convex
+          BVH/Convex object of source mesh
         """
-        bvh = mesh_to_BVH(mesh)
-        return bvh
+
+        if mesh.is_convex:
+            obj = mesh_to_convex(mesh)
+        else:
+            obj = mesh_to_BVH(mesh)
+        return obj
 
     def _extract_name(self, geom):
         """
@@ -685,6 +725,25 @@ def mesh_to_BVH(mesh):
     return bvh
 
 
+def mesh_to_convex(mesh):
+    """
+    Create a Convex object from a Trimesh object
+
+    Parameters
+    -----------
+    mesh : Trimesh
+      Input geometry
+
+    Returns
+    ------------
+    convex : fcl.Convex
+      Convex of input geometry
+    """
+    fs = np.concatenate((3 * np.ones((len(mesh.faces), 1), dtype=np.int64), mesh.faces),
+                        axis=1)
+    return fcl.Convex(mesh.vertices, len(fs), fs.flatten())
+
+
 def scene_to_collision(scene):
     """
     Create collision objects from a trimesh.Scene object.
