@@ -1,5 +1,6 @@
 import numpy as np
 import collections
+import uuid
 
 from . import cameras
 from . import lighting
@@ -10,7 +11,6 @@ from .. import convex
 from .. import caching
 from .. import grouping
 from .. import transformations
-from .. import bounds as bounds_module
 
 from ..util import unique_name
 from ..exchange import export
@@ -242,10 +242,10 @@ class Scene(Geometry3D):
         # avoid accessing attribute in tight loop
         geometry = self.geometry
         # start with the last modified time of the scene graph
-        hashable = [hex(hash(self.graph))]
+        hashable = [hex(self.graph.transforms.__hash__())]
         # take the re-hex string of the hash
         hashable.extend(hex(geometry[k].__hash__()) for k in
-                        sorted(geometry.keys()))
+                        geometry.keys())
         return caching.hash_fast(
             ''.join(hashable).encode('utf-8'))
 
@@ -290,42 +290,43 @@ class Scene(Geometry3D):
     @caching.cache_decorator
     def bounds_corners(self):
         """
-        A list of points that represent the corners of the
-        AABB of every geometry in the scene.
-
-        This can be useful if you want to take the AABB in
-        a specific frame.
+        Get the post-transform AABB for each node
+        which has geometry defined.
 
         Returns
         -----------
-        corners: (n, 3) float
-          Points in space
+        corners : dict
+          Bounds for each node with vertices:
+           {node_name : (2, 3) float}
         """
-        # the saved corners of each instance
-        corners_inst = []
-        # (n, 3) float corners of each geometry
-        corners_geom = {k: bounds_module.corners(v.bounds)
-                        for k, v in self.geometry.items()
-                        if v.bounds is not None}
-        if len(corners_geom) == 0:
-            return np.array([])
+        # collect AABB for each geometry
+        corners = {}
+        # collect vertices for every mesh
+        vertices = {k: m.vertices for k, m in
+                    self.geometry.items()
+                    if hasattr(m, 'vertices') and
+                    len(m.vertices) > 0}
+        # handle 2D geometries
+        vertices.update(
+            {k: np.column_stack((v, np.zeros(len(v))))
+             for k, v in vertices.items() if v.shape[1] == 2})
 
+        # loop through every node with geometry
         for node_name in self.graph.nodes_geometry:
             # access the transform and geometry name from node
             transform, geometry_name = self.graph[node_name]
-            # not all nodes have associated geometry
-            if geometry_name not in corners_geom:
+            # will be None if no vertices for this node
+            points = vertices.get(geometry_name)
+            # skip empty geometries
+            if points is None:
                 continue
-            # transform geometry corners into where
-            # the instance of the geometry is located
-            corners_inst.extend(
-                transformations.transform_points(
-                    corners_geom[geometry_name],
-                    transform))
-        # make corners numpy array
-        corners_inst = np.array(corners_inst,
-                                dtype=np.float64)
-        return corners_inst
+            # apply just the rotation to skip N multiplies
+            dot = np.dot(transform[:3, :3], points.T)
+            # append the AABB with translation applied after
+            corners[node_name] = np.array(
+                [dot.min(axis=1) + transform[:3, 3],
+                 dot.max(axis=1) + transform[:3, 3]])
+        return corners
 
     @caching.cache_decorator
     def bounds(self):
@@ -338,12 +339,14 @@ class Scene(Geometry3D):
           Position of [min, max] bounding box
           Returns None if no valid bounds exist
         """
-        corners = self.bounds_corners
-        if len(corners) == 0:
+        bounds_corners = self.bounds_corners
+        if len(bounds_corners) == 0:
             return None
-        bounds = np.array([corners.min(axis=0),
-                           corners.max(axis=0)])
-        return bounds
+        # combine each geometry node AABB into a larger list
+        corners = np.vstack(list(self.bounds_corners.values()))
+        return np.array([corners.min(axis=0),
+                         corners.max(axis=0)],
+                        dtype=np.float64)
 
     @caching.cache_decorator
     def extents(self):
@@ -564,7 +567,7 @@ class Scene(Geometry3D):
 
         rotation = transformations.euler_matrix(*angles)
         transform = cameras.look_at(
-            self.bounds_corners,
+            self.bounds,
             fov=fov,
             rotation=rotation,
             distance=distance,
@@ -800,7 +803,9 @@ class Scene(Geometry3D):
         ---------
         hull: Trimesh object, convex hull of all meshes in scene
         """
-        points = util.vstack_empty([m.vertices for m in self.dump()])
+        points = util.vstack_empty(
+            [m.vertices
+             for m in self.dump()])
         hull = convex.convex_hull(points)
         return hull
 
@@ -977,7 +982,7 @@ class Scene(Geometry3D):
 
         Parameters
         -----------
-        scale : float
+        scale : float or (3,) float
           Factor to scale meshes and transforms
 
         Returns
@@ -985,52 +990,115 @@ class Scene(Geometry3D):
         scaled : trimesh.Scene
           A copy of the current scene but scaled
         """
-        scale = float(scale)
-        # matrix for 2D scaling
-        scale_2D = np.eye(3) * scale
-        # matrix for 3D scaling
-        scale_3D = np.eye(4) * scale
+        # convert 2D geometries to 3D for 3D scaling factors
+        scale_is_3D = isinstance(scale, (list, tuple, np.ndarray)) and len(scale) == 3
 
-        # preallocate transforms and geometries
-        nodes = np.array(self.graph.nodes_geometry)
-        transforms = np.zeros((len(nodes), 4, 4))
-        geometries = [None] * len(nodes)
-
-        # collect list of transforms
-        for i, node in enumerate(nodes):
-            transforms[i], geometries[i] = self.graph[node]
+        if scale_is_3D and np.all(np.asarray(scale) == scale[0]):
+            # scale is uniform
+            scale = float(scale[0])
+            scale_is_3D = False
+        elif not scale_is_3D:
+            scale = float(scale)
 
         # result is a copy
         result = self.copy()
-        # remove all existing transforms
-        result.graph.clear()
 
-        for group in grouping.group(geometries):
-            # hashable reference to self.geometry
-            geometry = geometries[group[0]]
-            # original transform from world to geometry
-            original = transforms[group[0]]
-            # transform for geometry
-            new_geom = np.dot(scale_3D, original)
+        if scale_is_3D:
+            # Copy all geometries that appear multiple times in the scene,
+            # such that no two nodes share the same geometry.
+            # This is required since the non-uniform scaling will most likely
+            # affect the same geometry in different poses differently.
+            # Note, that this is not needed in the case of uniform scaling.
+            for geom_name in result.graph.geometry_nodes:
+                nodes_with_geom = result.graph.geometry_nodes[geom_name]
+                if len(nodes_with_geom) > 1:
+                    geom = result.geometry[geom_name]
+                    for n in nodes_with_geom:
+                        p = result.graph.transforms.parents[n]
+                        result.add_geometry(
+                            geometry=geom.copy(),
+                            geom_name=geom_name,
+                            node_name=n,
+                            parent_node_name=p,
+                            transform=result.graph.transforms.edge_data[(
+                                p, n)].get('matrix', None),
+                            extras=result.graph.transforms.edge_data[(
+                                p, n)].get('extras', None),
+                        )
+                    result.delete_geometry(geom_name)
 
-            if result.geometry[geometry].vertices.shape[1] == 2:
-                # if our scene is 2D only scale in 2D
-                result.geometry[geometry].apply_transform(scale_2D)
-            else:
-                # otherwise apply the full transform
-                result.geometry[geometry].apply_transform(new_geom)
+            # Convert all 2D paths to 3D paths
+            for geom_name in result.geometry:
+                if result.geometry[geom_name].vertices.shape[1] == 2:
+                    result.geometry[geom_name] = result.geometry[geom_name].to_3D()
 
-            for node, T in zip(nodes[group],
-                               transforms[group]):
-                # generate the new transforms
-                transform = util.multi_dot(
-                    [scale_3D, T, np.linalg.inv(new_geom)])
-                # apply scale to translation
-                transform[:3, 3] *= scale
-                # update scene with new transforms
-                result.graph.update(frame_to=node,
-                                    matrix=transform,
-                                    geometry=geometry)
+            # Scale all geometries by un-doing their local rotations first
+            for key in result.graph.nodes_geometry:
+                T, geom_name = result.graph.get(key)
+                T[:3, 3] = 0.0
+
+                # Get geometry transform w.r.t. base frame
+                result.geometry[geom_name].apply_transform(T).apply_scale(
+                    scale).apply_transform(np.linalg.inv(T))
+
+            # Scale all transformations in the scene graph
+            edge_data = result.graph.transforms.edge_data
+            for uv in edge_data:
+                if 'matrix' in edge_data[uv]:
+                    props = edge_data[uv]
+                    T = edge_data[uv]['matrix']
+                    T[:3, 3] *= scale
+                    props['matrix'] = T
+                    result.graph.update(frame_from=uv[0], frame_to=uv[1], **props)
+
+            # Clear cache
+            result.graph.transforms._cache = {}
+            result.graph.transforms._modified = str(uuid.uuid4())
+            result.graph._cache.clear()
+        else:
+            # matrix for 2D scaling
+            scale_2D = np.eye(3) * scale
+            # matrix for 3D scaling
+            scale_3D = np.eye(4) * scale
+
+            # preallocate transforms and geometries
+            nodes = np.array(self.graph.nodes_geometry)
+            transforms = np.zeros((len(nodes), 4, 4))
+            geometries = [None] * len(nodes)
+
+            # collect list of transforms
+            for i, node in enumerate(nodes):
+                transforms[i], geometries[i] = self.graph[node]
+
+            # remove all existing transforms
+            result.graph.clear()
+
+            for group in grouping.group(geometries):
+                # hashable reference to self.geometry
+                geometry = geometries[group[0]]
+                # original transform from world to geometry
+                original = transforms[group[0]]
+                # transform for geometry
+                new_geom = np.dot(scale_3D, original)
+
+                if result.geometry[geometry].vertices.shape[1] == 2:
+                    # if our scene is 2D only scale in 2D
+                    result.geometry[geometry].apply_transform(scale_2D)
+                else:
+                    # otherwise apply the full transform
+                    result.geometry[geometry].apply_transform(new_geom)
+
+                for node, T in zip(nodes[group],
+                                   transforms[group]):
+                    # generate the new transforms
+                    transform = util.multi_dot(
+                        [scale_3D, T, np.linalg.inv(new_geom)])
+                    # apply scale to translation
+                    transform[:3, 3] *= scale
+                    # update scene with new transforms
+                    result.graph.update(frame_to=node,
+                                        matrix=transform,
+                                        geometry=geometry)
         return result
 
     def copy(self):
