@@ -58,7 +58,8 @@ class ColorVisuals(Visuals):
         """
         self.mesh = mesh
         self._data = caching.DataStore()
-        self._cache = caching.Cache(id_function=self.crc)
+        self._cache = caching.Cache(
+            id_function=self._data.__hash__)
 
         self.defaults = {
             'material_diffuse': np.array([102, 102, 102, 255],
@@ -122,7 +123,7 @@ class ColorVisuals(Visuals):
             return None
 
         # do bookkeeping
-        self._verify_crc()
+        self._verify_hash()
 
         # check modes in data
         if 'vertex_colors' in self._data:
@@ -132,23 +133,8 @@ class ColorVisuals(Visuals):
 
         return None
 
-    def crc(self):
-        """
-        A checksum for the current visual object and its parent mesh.
-
-        Returns
-        ----------
-        crc : int
-          Checksum of data in visual object and its parent mesh
-        """
-        # will make sure everything has been transferred
-        # to datastore that needs to be before returning crc
-
-        result = self._data.fast_hash()
-        if hasattr(self.mesh, 'crc'):
-            # bitwise xor combines hashes better than a sum
-            result ^= self.mesh.crc()
-        return result
+    def __hash__(self):
+        return self._data.__hash__()
 
     def copy(self):
         """
@@ -276,8 +262,8 @@ class ColorVisuals(Visuals):
         when requested.
         * If colors have never been set, a (count,4) tiled copy of the default diffuse
         color will be stored in the cache
-        ** the CRC on creation for these cached default colors will also be stored
-        ** if the cached color array is altered (different CRC than when it was
+        ** the hash on creation for these cached default colors will also be stored
+        ** if the cached color array is altered (different hash than when it was
         created) we consider that now to be user data and the array is moved from
         the cache to the DataStore.
 
@@ -303,8 +289,8 @@ class ColorVisuals(Visuals):
 
         # the face or vertex colors
         key_colors = str(name) + '_colors'
-        # the initial crc of the
-        key_crc = key_colors + '_crc'
+        # the initial hash of the colors
+        key_hash = key_colors + '_hash'
 
         if key_colors in self._data:
             # if a user has explicitly stored or changed the color it
@@ -317,7 +303,7 @@ class ColorVisuals(Visuals):
             colors = self._cache[key_colors]
             # if the cached colors have been changed since creation we move
             # them to data
-            if colors.crc() != self._cache[key_crc]:
+            if hash(colors) != self._cache[key_hash]:
                 # call the setter on the property using exec
                 # this avoids having to pass a setter to this function
                 if name == 'face':
@@ -353,15 +339,15 @@ class ColorVisuals(Visuals):
                 colors.shape != (count, 4)):
             raise ValueError('face colors incorrect shape!')
 
-        # subclass the array to track for changes using a CRC
+        # subclass the array to track for changes using a hash
         colors = caching.tracked_array(colors)
         # put the generated colors and their initial checksum into cache
         self._cache[key_colors] = colors
-        self._cache[key_crc] = colors.crc()
+        self._cache[key_hash] = hash(colors)
 
         return colors
 
-    def _verify_crc(self):
+    def _verify_hash(self):
         """
         Verify the checksums of cached face and vertex color, to verify
         that a user hasn't altered them since they were generated from
@@ -377,8 +363,8 @@ class ColorVisuals(Visuals):
         for name in ['face', 'vertex']:
             # the face or vertex colors
             key_colors = str(name) + '_colors'
-            # the initial crc of the
-            key_crc = key_colors + '_crc'
+            # the initial hash of the colors
+            key_hash = key_colors + '_hash'
 
             if key_colors not in self._cache:
                 continue
@@ -386,7 +372,7 @@ class ColorVisuals(Visuals):
             colors = self._cache[key_colors]
             # if the cached colors have been changed since creation
             # move them to data
-            if colors.crc() != self._cache[key_crc]:
+            if hash(colors) != self._cache[key_hash]:
                 if name == 'face':
                     self.face_colors = colors
                 elif name == 'vertex':
@@ -469,7 +455,8 @@ class ColorVisuals(Visuals):
 
     def concatenate(self, other, *args):
         """
-        Concatenate two or more ColorVisuals objects into a single object.
+        Concatenate two or more ColorVisuals objects
+        into a single object.
 
         Parameters
         -----------
@@ -479,8 +466,9 @@ class ColorVisuals(Visuals):
 
         Returns
         -----------
-        result: ColorVisuals object containing information from current
-                object and others in the order it was passed.
+        result : ColorVisuals
+          Containing information from current
+          object and others in the order it was passed.
         """
         # avoid a circular import
         from . import objects
@@ -519,9 +507,6 @@ class VertexColor(Visuals):
     def kind(self):
         return 'vertex'
 
-    def crc(self):
-        return self._colors.crc()
-
     def update_vertices(self, mask):
         if self._colors is not None:
             self._colors = self._colors[mask]
@@ -553,7 +538,8 @@ class VertexColor(Visuals):
 
     def concatenate(self, other):
         """
-        Concatenate this visual object with another VertexVisuals.
+        Concatenate this visual object with another
+        VertexVisuals.
 
         Parameters
         -----------
@@ -569,6 +555,9 @@ class VertexColor(Visuals):
             self.vertex_colors,
             other.vertex_colors))
 
+    def __hash__(self):
+        return self._colors.__hash__()
+
 
 def to_rgba(colors, dtype=np.uint8):
     """
